'****************************************************************************
' Developed & Written by Greg Cook --- August 24, 2000
' Email: greg_cook@bigoot.com
' Web site: http://www.bigfoot.com/~greg_cook
'
' PURPOSE:
'
' Remove the read-only attribute of all the files and folders
' within a specified folder, or make them all read-only.
'
' USAGE:
'
' Double-click it to run, and you will be prompted for the path and for what to
' do to the files. Or you can run it in a "short-cut," supplying the path and
' what to do as "arguments."
'
' <myname>.vbs arg1 arg2
'
' arg1 = Source folder (folder to start in)
' arg2 = True (remove read-only) or empty (make read-only)
'
' DISCLAIMER:
'
' This script is FREE for your use, so please use and share with others
' as you see fit.
'
' This software is provided AS IS without warranty of any kind including but
' not limited to the implied warranties of merchantability and fitness for a
' particular purpose. The publisher assumes no liability for any alleged or
' actual damages arising from the use of this program. (Some states do not
' allow the exclusion of implied warranties, so the exclusion may not apply
' to you.)
'
' If you are interested in learning about VBScript (or JavaScript) check out:
' http://msdn.microsoft.com/scripting/
'
'****************************************************************************

'****************************************************************************
'Clear memory before exiting
'****************************************************************************
Sub Cleanup()
  'Close files that were opened
  objLogFile.Close
  objStatusFile.Close
  Err.Clear
  'Clear object variables
  Set objLogFile = Nothing
  Set objStatusFile = Nothing
  Set objArgs = Nothing
  Set shell = Nothing
  Set fso = Nothing
End Sub

'****************************************************************************
'Standard forced exit procedure
'****************************************************************************
Sub ExitMeNow(strWhyExit)
  On Error Resume Next
  'Write why the exit is occurring
  WriteLog strWhyExit
  MsgBox strWhyExit, vbCritical, "Read-only Error"
  Cleanup
  'Quit running this script
  WScript.Quit
End Sub

'****************************************************************************
'Check each file and reset it if it is read-only
'****************************************************************************
Sub MakeReadOnly(objFileOrFolder, blnMakeReadOnly)
  'Bypass all errors
  On Error Resume Next
  With objFileOrFolder
    If .Attributes And 1 Then
      If Not blnMakeReadOnly Then
        .Attributes = .Attributes - 1
        WriteLog .Path
        lngFileCount = lngFileCount + 1
      End If
    Else
      If blnMakeReadOnly Then
        .Attributes = .Attributes + 1
        WriteLog .Path
        lngFileCount = lngFileCount + 1
      End If
    End If
  End With
  Err.Clear
End Sub

'****************************************************************************
' Check each of the files in the specified folder
'****************************************************************************
Sub DoFiles(folderspec)  
  'Bypass all errors
  On Error Resume Next

  Dim f, fc, f1

  With fso
    'Reference the specified folder
    Set f = .GetFolder(folderspec)
    'Get list of files
    Set fc = f.Files

    'Loop through each file
    'Check for success, and write to log file
    'Increment file count
    For Each f1 In fc
      MakeReadOnly f1, blnMakeReadOnly
      Err.Clear
    Next
  End With
  Set f = Nothing
  Set fc = Nothing
End Sub

'****************************************************************************
'Check the specified folder and all items in all sub-folders
'****************************************************************************
Sub DoFolders(folderspec)  
  'Bypass all errors
  On Error Resume Next

  Dim f, f1, sf

  DoFiles(folderspec)
  Err.Clear
  Set f = fso.GetFolder(folderspec)  
  Set sf = f.SubFolders

  MakeReadOnly f, blnMakeReadOnly

  For Each f1 In sf
    WriteStatus "Checking " & f1.Path
    DoFolders(f1.path)
    Err.Clear
  Next
  Set f = Nothing
  Set sf = Nothing
End Sub

'****************************************************************************
'Write the specified string to the log file
'****************************************************************************
Sub WriteLog(strStatus)
  'Bypass all errors
  On Error Resume Next
  objLogFile.Write strStatus & vbNewLine
End Sub

'****************************************************************************
'Write the specified string to the status file
'****************************************************************************
Sub WriteStatus(strStatus)
  'Bypass all errors
  On Error Resume Next
  objStatusFile.Write strStatus & "..." & vbNewLine
End Sub

'****************************************************************************
'Return only the name portion of the specified file
'****************************************************************************
Function NameFromFile(strFilename)
  On Error Resume Next
  NameFromFile = StrReverse(strFilename)
  NameFromFile = StrReverse(Mid(NameFromFile, 1, InStr(1, NameFromFile, "\") - 1))
End Function

'****************************************************************************
'Return a string with of the extension of the specified filename
'****************************************************************************
Function ExtFromFile(strFilename)
  On Error Resume Next
  Dim intPos
  ExtFromFile = StrReverse(strFilename)
  ExtFromFile = StrReverse(Mid(ExtFromFile, 1, InStr(1, ExtFromFile, ".") - 1))
End Function

'****************************************************************************
'Return a string with the extension removed from the specified filename
'****************************************************************************
Function NoExtension(strFilename)
  On Error Resume Next
  Dim intPos
  NoExtension = StrReverse(strFilename)
  NoExtension = StrReverse(Mid(NoExtension, InStr(1, NoExtension, ".") + 1))
End Function

'****************************************************************************
'Return a string converted to "Proper case"
'****************************************************************************
Function MakeProperCase(strString)
  On Error Resume Next
  MakeProperCase = UCase(Left(strString, 1)) & LCase(Mid(strString, 2))
End Function

'****************************************************************************
' Main
'****************************************************************************
'Bypass all errors
On Error Resume Next

Dim shell, fso, objLogFile, objStatusFile, objArgs, strFolder
Dim strTempPath, strLogFileName, strStatusFileName, strComplete
Dim lngFileCount, intArg
Dim blnMakeReadOnly

'Get the name of this script file
strMyName = NameFromFile(WScript.ScriptFullName)
strMyName = NoExtension(strMyName)
strMyName = MakeProperCase(strMyName) & " script"
'Prompt user to insure that they know what is about to happen
If MsgBox("This script will add/remove the read-only attributes of all the files in a specified folder." & vbNewLine & vbNewLine & "Do you wish to continue?", vbCritical + vbYesNo, strMyName) = vbNo Then
  WScript.Quit
End If

'Reference the shell obejct
Set shell = CreateObject("WScript.Shell")
'Reference the file system object
Set fso = CreateObject("Scripting.FileSystemObject")

'Path of the temp folder
strTempPath = shell.ExpandEnvironmentStrings("%TEMP%") & "\"
'Get the name of this file and append ".txt" to the end of it
'because it will be used to log and status
strLogFileName = strTempPath & strMyName & "_log.txt"
strStatusFileName = strTempPath & strMyName & "_status.txt"

'Open the text file for writing
With fso
  Set objLogFile = .OpenTextFile(strLogFileName, 2, True)
  Set objStatusFile = .OpenTextFile(strStatusFileName, 2, True)
End With

WriteStatus "Initializing " & Now()
'Read arguments into object variable
Set objArgs = WScript.Arguments
If objArgs.Count > 0 Then
  strFolder = objArgs(0)
  If objArgs.Count > 1 Then
    blnMakeReadOnly = objArgs(1)
  Else
    blnMakeReadOnly = False
  End If
Else
  strFolder = vbNullString
  blnMakeReadOnly = False
End If

'Initialize the variables
lngFileCount = 0

WriteLog "Checking source variable"
'If the string has no length, ask the user for it
If Len(strFolder) = 0 Then
  WriteLog "Prompting user for source"
  strFolder = InputBox("Source folder?", strMyName)
  'If the string has no length, the user cancelled
  If Len(strFolder) = 0 Then
    ExitMeNow "Source not set. User cancelled"
  End If
End If

'Remove backslash, if it has one as last character
If Right(strFolder, 1) = "\" Then
  strFolder = Left(strFolder, Len(strFolder) - 1)
End If

'See if the folder exists
If fso.FolderExists(strFolder) Then
  If Not blnMakeReadOnly Then
    blnMakeReadOnly = (MsgBox("Make read-only?", vbYesNo + vbQuestion, strMyName) = vbYes)
  End If
  DoFolders strFolder & "\"
Else
  ExitMeNow strFolder & " is missing!"
End If

strComplete = lngFileCount & " files were altered!"
WriteLog strComplete
If lngFileCount = 0 Then
  MsgBox strComplete, vbInformation, strMyName
  fso.DeleteFile strLogFileName, True
  Err.Clear
Else
  MsgBox strComplete & vbNewLine & vbNewLine & "Please see " & strLogFileName & " for list of affected files.", vbInformation, strMyName
End If

Cleanup
